var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import React, { useState } from 'react';
import { UUID } from '@lumino/coreutils';
import { closeIcon } from '@jupyterlab/ui-components';
const SETTING_NAME = 'language_servers';
const SERVER_SETTINGS = 'serverSettings';
function debounce(func, timeout = 500) {
    let timer;
    return (...args) => {
        clearTimeout(timer);
        timer = setTimeout(() => {
            func(...args);
        }, timeout);
    };
}
function BuildSettingForm(props) {
    const _a = props.schema, _b = SERVER_SETTINGS, serverSettingsSchema = _a[_b], otherSettingsSchema = __rest(_a, [typeof _b === "symbol" ? _b : _b + ""]);
    const _c = props.settings, _d = SERVER_SETTINGS, serverSettings = _c[_d], { serverName } = _c, otherSettings = __rest(_c, [typeof _d === "symbol" ? _d : _d + "", "serverName"]);
    const [currentServerName, setCurrentServerName] = useState(serverName);
    const onServerNameChange = (e) => {
        props.updateSetting(props.serverHash, { serverName: e.target.value });
        setCurrentServerName(e.target.value);
    };
    const serverSettingWithType = {};
    Object.entries(serverSettings).forEach(([key, value]) => {
        const newProps = {
            property: key,
            type: typeof value,
            value
        };
        serverSettingWithType[UUID.uuid4()] = newProps;
    });
    const [propertyMap, setPropertyMap] = useState(serverSettingWithType);
    const defaultOtherSettings = {};
    Object.entries(otherSettingsSchema).forEach(([key, value]) => {
        if (key in otherSettings) {
            defaultOtherSettings[key] = otherSettings[key];
        }
        else {
            defaultOtherSettings[key] = value['default'];
        }
    });
    const [otherSettingsComposite, setOtherSettingsComposite] = useState(defaultOtherSettings);
    const onOtherSettingsChange = (property, value, type) => {
        let settingValue = value;
        if (type === 'number') {
            settingValue = parseFloat(value);
        }
        const newProps = Object.assign(Object.assign({}, otherSettingsComposite), { [property]: settingValue });
        props.updateSetting(props.serverHash, newProps);
        setOtherSettingsComposite(newProps);
    };
    const addProperty = () => {
        const hash = UUID.uuid4();
        const newMap = Object.assign(Object.assign({}, propertyMap), { [hash]: { property: '', type: 'string', value: '' } });
        const payload = {};
        Object.values(newMap).forEach(value => {
            payload[value.property] = value.value;
        });
        props.updateSetting(props.serverHash, { [SERVER_SETTINGS]: payload });
        setPropertyMap(newMap);
    };
    const removeProperty = (entryHash) => {
        const newMap = {};
        Object.entries(propertyMap).forEach(([hash, value]) => {
            if (hash !== entryHash) {
                newMap[hash] = value;
            }
            const payload = {};
            Object.values(newMap).forEach(value => {
                payload[value.property] = value.value;
            });
            props.updateSetting(props.serverHash, { [SERVER_SETTINGS]: payload });
            setPropertyMap(newMap);
        });
    };
    const setProperty = (hash, property) => {
        if (hash in propertyMap) {
            const newMap = Object.assign(Object.assign({}, propertyMap), { [hash]: property });
            const payload = {};
            Object.values(newMap).forEach(value => {
                payload[value.property] = value.value;
            });
            setPropertyMap(newMap);
            props.updateSetting(props.serverHash, { [SERVER_SETTINGS]: payload });
        }
    };
    return (React.createElement("div", { className: "array-item" },
        React.createElement("div", { className: "form-group " },
            React.createElement("div", { className: "jp-FormGroup-content" },
                React.createElement("div", { className: "jp-objectFieldWrapper" },
                    React.createElement("fieldset", null,
                        React.createElement("div", { className: "form-group small-field" },
                            React.createElement("div", { className: "jp-modifiedIndicator jp-errorIndicator" }),
                            React.createElement("div", { className: "jp-FormGroup-content" },
                                React.createElement("h3", { className: "jp-FormGroup-fieldLabel jp-FormGroup-contentItem" }, "Server name:"),
                                React.createElement("div", { className: "jp-inputFieldWrapper jp-FormGroup-contentItem" },
                                    React.createElement("input", { className: "form-control", type: "text", required: true, value: currentServerName, onChange: e => {
                                            onServerNameChange(e);
                                        } })),
                                React.createElement("div", { className: "validationErrors" },
                                    React.createElement("div", null,
                                        React.createElement("ul", { className: "error-detail bs-callout bs-callout-info" },
                                            React.createElement("li", { className: "text-danger" }, "is a required property")))))),
                        Object.entries(otherSettingsSchema).map(([property, value], idx) => {
                            return (React.createElement("div", { key: `${idx}-${property}`, className: "form-group small-field" },
                                React.createElement("div", { className: "jp-FormGroup-content" },
                                    React.createElement("h3", { className: "jp-FormGroup-fieldLabel jp-FormGroup-contentItem" }, value.title),
                                    React.createElement("div", { className: "jp-inputFieldWrapper jp-FormGroup-contentItem" },
                                        React.createElement("input", { className: "form-control", placeholder: "", type: value.type, value: otherSettingsComposite[property], onChange: e => onOtherSettingsChange(property, e.target.value, value.type) })),
                                    React.createElement("div", { className: "jp-FormGroup-description" }, value.description),
                                    React.createElement("div", { className: "validationErrors" }))));
                        }),
                        React.createElement("fieldset", null,
                            React.createElement("legend", null, serverSettingsSchema['title']),
                            Object.entries(propertyMap).map(([hash, property]) => {
                                return (React.createElement(PropertyFrom, { key: hash, hash: hash, property: property, removeProperty: removeProperty, setProperty: debounce(setProperty) }));
                            }),
                            React.createElement("span", null, serverSettingsSchema['description'])))))),
        React.createElement("div", { className: "jp-ArrayOperations" },
            React.createElement("button", { className: "jp-mod-styled jp-mod-reject", onClick: addProperty }, props.trans.__('Add property')),
            React.createElement("button", { className: "jp-mod-styled jp-mod-warn jp-FormGroup-removeButton", onClick: () => props.removeSetting(props.serverHash) }, props.trans.__('Remove server')))));
}
function PropertyFrom(props) {
    const [state, setState] = useState(Object.assign({}, props.property));
    const TYPE_MAP = { string: 'text', number: 'number', boolean: 'checkbox' };
    const removeItem = () => {
        props.removeProperty(props.hash);
    };
    const changeName = (newName) => {
        const newState = Object.assign(Object.assign({}, state), { property: newName });
        props.setProperty(props.hash, newState);
        setState(newState);
    };
    const changeValue = (newValue, type) => {
        let value = newValue;
        if (type === 'number') {
            value = parseFloat(newValue);
        }
        const newState = Object.assign(Object.assign({}, state), { value });
        props.setProperty(props.hash, newState);
        setState(newState);
    };
    const changeType = (newType) => {
        let value;
        if (newType === 'boolean') {
            value = false;
        }
        else if (newType === 'number') {
            value = 0;
        }
        else {
            value = '';
        }
        const newState = Object.assign(Object.assign({}, state), { type: newType, value });
        setState(newState);
        props.setProperty(props.hash, newState);
    };
    return (React.createElement("div", { key: props.hash, className: "form-group small-field" },
        React.createElement("div", { className: "jp-FormGroup-content" },
            React.createElement("input", { style: { marginRight: '25px' }, className: "form-control", type: "text", required: true, placeholder: 'Property name', value: state.property, onChange: e => {
                    changeName(e.target.value);
                } }),
            React.createElement("select", { style: { marginRight: '25px' }, className: "form-control", value: state.type, onChange: e => changeType(e.target.value) },
                React.createElement("option", { value: "string" }, "String"),
                React.createElement("option", { value: "number" }, "Number"),
                React.createElement("option", { value: "boolean" }, "Boolean")),
            React.createElement("input", { style: { marginRight: '25px' }, className: "form-control", type: TYPE_MAP[state.type], required: false, placeholder: 'Property value', value: state.type !== 'boolean' ? state.value : undefined, checked: state.type === 'boolean' ? state.value : undefined, onChange: state.type !== 'boolean'
                    ? e => changeValue(e.target.value, state.type)
                    : e => changeValue(e.target.checked, state.type) }),
            React.createElement("button", { className: "jp-mod-minimal jp-Button", onClick: removeItem },
                React.createElement(closeIcon.react, null)))));
}
class SettingRenderer extends React.Component {
    constructor(props) {
        super(props);
        this.removeSetting = (hash) => {
            if (hash in this.state.items) {
                const items = {};
                for (const key in this.state.items) {
                    if (key !== hash) {
                        items[key] = this.state.items[key];
                    }
                }
                this.setState(old => {
                    return Object.assign(Object.assign({}, old), { items });
                }, () => {
                    this.saveServerSetting();
                });
            }
        };
        this.updateSetting = (hash, newSetting) => {
            if (hash in this.state.items) {
                const items = {};
                for (const key in this.state.items) {
                    if (key === hash) {
                        items[key] = Object.assign(Object.assign({}, this.state.items[key]), newSetting);
                    }
                    else {
                        items[key] = this.state.items[key];
                    }
                }
                this.setState(old => {
                    return Object.assign(Object.assign({}, old), { items });
                }, () => {
                    this.saveServerSetting();
                });
            }
        };
        this.addServerSetting = () => {
            let index = 0;
            let key = 'newKey';
            while (Object.values(this.state.items)
                .map(val => val.serverName)
                .includes(key)) {
                index += 1;
                key = `newKey-${index}`;
            }
            this.setState(old => (Object.assign(Object.assign({}, old), { items: Object.assign(Object.assign({}, old.items), { [UUID.uuid4()]: Object.assign(Object.assign({}, this._defaultSetting), { serverName: key }) }) })), () => {
                this.saveServerSetting();
            });
        };
        this.saveServerSetting = () => {
            const settings = {};
            Object.values(this.state.items).forEach(item => {
                const { serverName } = item, setting = __rest(item, ["serverName"]);
                settings[serverName] = setting;
            });
            this._setting.set(SETTING_NAME, settings).catch(console.error);
        };
        this._setting = props.formContext.settings;
        this._trans = props.translator.load('jupyterlab');
        const schema = this._setting.schema['definitions'];
        this._defaultSetting = schema['language-server']['default'];
        this._schema = schema['language-server']['properties'];
        const title = props.schema.title;
        const desc = props.schema.description;
        const settings = props.formContext.settings;
        const compositeData = settings.get(SETTING_NAME).composite;
        let items = {};
        if (compositeData) {
            Object.entries(compositeData).forEach(([key, value]) => {
                if (value) {
                    const hash = UUID.uuid4();
                    items[hash] = Object.assign({ serverName: key }, value);
                }
            });
        }
        this.state = { title, desc, items };
    }
    render() {
        return (React.createElement("div", null,
            React.createElement("fieldset", null,
                React.createElement("legend", null, this.state.title),
                React.createElement("p", { className: "field-description" }, this.state.desc),
                React.createElement("div", { className: "field field-array field-array-of-object" }, Object.entries(this.state.items).map(([hash, value], idx) => {
                    return (React.createElement(BuildSettingForm, { key: `${idx}-${hash}`, trans: this._trans, removeSetting: this.removeSetting, updateSetting: debounce(this.updateSetting), serverHash: hash, settings: value, schema: this._schema }));
                })),
                React.createElement("div", null,
                    React.createElement("button", { style: { margin: 2 }, className: "jp-mod-styled jp-mod-reject", onClick: this.addServerSetting }, this._trans.__('Add server'))))));
    }
}
/**
 * Custom setting renderer for language server.
 */
export function renderServerSetting(props, translator) {
    return React.createElement(SettingRenderer, Object.assign({}, props, { translator: translator }));
}
//# sourceMappingURL=renderer.js.map